@@ -1,8 +1,8 @@
 package Sim::OPT::Sim;
-# Copyright (C) 2008-2014 by Gian Luca Brunetti and Politecnico di Milano.
+# Copyright (C) 2008-2015 by Gian Luca Brunetti and Politecnico di Milano.
 # This is the module Sim::OPT::Sim of Sim::OPT, a program for detailed metadesign managing parametric explorations through the ESP-r building performance simulation platform and performing optimization by block coordinate descent.
 # This is free software.  You can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
-
+ 
 use v5.14;
 # use v5.20;
 use Exporter;
@@ -42,7 +42,7 @@ no warnings;
 
 our @EXPORT = qw( sim ); # our @EXPORT = qw( );
 
-$VERSION = '0.46.2'; # our $VERSION = '';
+$VERSION = '0.47'; # our $VERSION = '';
 
 
 #########################################################################################
@@ -86,14 +86,14 @@ sub sim    # This function launch the simulations in ESP-r
 	
 	$tee = new IO::Tee(\*STDOUT, ">>$toshell"); # GLOBAL ZZZ
 	
-	open( OUTFILE, ">>$outfile" ) or die "Can't open $outfile: $!"; 
+	#open( OUTFILE, ">>$outfile" ) or die "Can't open $outfile: $!"; 
 	open( TOSHELL, ">>$toshell" ) or die "Can't open $toshell: $!"; 
 	say  "\nNow in Sim::OPT::Sim.\n";
 	say TOSHELL "\n#Now in Sim::OPT::Sim.\n";
 	
 	%dowhat = %main::dowhat;
 
-	%simtitles = %main::simtitles; say "dumpINSIM(\@simtitles): " . dump(@simtitles);
+	%simtitles = %main::simtitles; say "dumpINSIM(\%simtitles): " . dump(%simtitles);
 	%retrievedata = %main::retrievedata;
 	@keepcolumns = @main::keepcolumns;
 	@weights = @main::weights;
@@ -156,7 +156,7 @@ sub sim    # This function launch the simulations in ESP-r
 		my @winneritems = @{ $d{winneritems} }; #say TOSHELL "dumpIN( \@winneritems) " . dump(@winneritems);
 		my $countvar = $d{countvar}; #say TOSHELL "dump(\$countvar): " . dump($countvar);
 		my $countstep = $d{countstep}; #say TOSHELL "dump(\$countstep): " . dump($countstep);						
-		my $to = $d{to}; #say TOSHELL "dump(\$to): " . dump($to);
+		my $to = $d{to}; say TOSHELL "dump(\$to): " . dump($to);
 		my $origin = $d{origin}; #say TOSHELL "dump(\$origin): " . dump($origin);
 		my @uplift = @{ $d{uplift} }; #say TOSHELL "dump(\@uplift): " . dump(@uplift);
 		#eval($getparshere);
@@ -220,7 +220,7 @@ sub sim    # This function launch the simulations in ESP-r
 						#{
 						#	if ( $countblock == 0 )
 						#	{
-								open( SIMLIST, ">>$simlist") or die;
+								open( SIMLIST, ">>$simlist") or die( "$!" );
 						#	}
 						#	else 
 						#	{
@@ -337,10 +337,11 @@ $printthis
 						$countsim++;
 					}
 				}
-				elsif ( $tooltype eq "energyplus" )
+				elsif ( ( $tooltype eq ( "generic" ) ) or ( $tooltype eq ( "energyplus" ) ) )
 				{	# TO DO: POSSIBILITY TO SPECIFY LINE AND ELEMENT OF TEXT SUBSTITUTIONS.
+					# THIS PART OF PROCEDURE HAS BEEN THOUGHT FOR ENERGYPLUS, THEN GENERALIZED.
 					my $countsim = 0;			
-					foreach my $simtitle_ref ( @simtitles )
+					foreach my $simtitle_ref ( @{ $simtitles{$counttool} } )
 					{
 						my $date_to_sim = $simtitle_ref->[0]; say TOSHELL "dump(\$date_to_sim ): " . dump( $date_to_sim ); 
 						my $begin = $simtitle_ref->[1]; say TOSHELL "dump(\$begin): " . dump($begin); 
@@ -349,33 +350,56 @@ $printthis
 						my $step = $simtitle_ref->[4]; say TOSHELL "dump(\$step): " . dump($step); 
 						 
 						my $epw = $simtitle_ref->[5]; say TOSHELL "dump(\$epw ): " . dump( $epw ); 
-						my $epwfile = $mypath . "/" . $epw; say TOSHELL"\$genfile " . dump( $genfile );
-						my $epoldfile = $simtitle_ref->[6]; say TOSHELL "dump(\$epoldfile ): " . dump( $epoldfile ); 
-						my $epnewfile = $simtitle_ref->[7]; say TOSHELL "dump(\$epnewfile ): " . dump( $epnewfile ); 
+						my $epwfile = $mypath . "/" . $epw; say TOSHELL"\$epwfile " . dump( $epwfile );
+						my $epdir = $simtitle_ref->[6]; say TOSHELL "dump(\$epdir ): " . dump( $epdir ); 
+						my $epoldfile = $simtitle_ref->[7]; say TOSHELL "dump(\$epoldfile ): " . dump( $epoldfile ); 
+						my $epnewfragment = $simtitle_ref->[8]; say TOSHELL "dump(\$epnewfragment ): " . dump( $epnewfragment );  
+						my $outputdir = $simtitle_ref->[9]; say TOSHELL "dump(\$outputdir ): " . dump( $outputdir ); 
+						my $modfiletype = $simtitle_ref->[10]; say TOSHELL "dump(\$modfiletype ): " . dump( $modfiletype ); 
+						my $resfiletype = $simtitle_ref->[11]; say TOSHELL "dump(\$resfiletype ): " . dump( $resfiletype ); 
+						my $epoldpath = $to . $epdir . "/" . $epoldfile; say TOSHELL "dump(\$epoldpath ): " . dump( $epoldpath ); 
+						my $tempname = $to;
+						$tempname =~ s/$mypath\/// ; say TOSHELL "dump(\$tempname ): " . dump( $tempname ); 
+						my $epnewfile = $tempname . $epnewfragment . "$modfiletype"; say TOSHELL "dump(\$epnewfile ): " . dump( $epnewfile ); 
+						my $epresroot = $tempname . $epnewfragment ; say TOSHELL "dump(\$epnewfile ): " . dump( $epnewfile ); 
+						my $epnewpath = $to . $epdir . "/" . $epnewfile; say TOSHELL "dump(\$epnewpath ): " . dump( $epnewpath ); 
 						my @simdos = @{ $simtitle_ref };
-						my @changes = @simdos[ 8..$#simdos ]; say TOSHELL "dump(\$epnewfile ): " . dump( $epnewfile ); 
-						
-						open ( EPOLDFILE, "$epoldfile" ) or die;
-						my @sourcecontents = <EPOLDFILE>;
-						close EPOLDFILE; say TOSHELL "dump(\@sourcecontents ): " . dump( @sourcecontents ); 
-						
-						open ( EPNEWFILE, ">$epnewfile" ) or die;
-						foreach my $row ( @sourcecontents )
+						my @changes = @simdos[ 12..$#simdos ]; say TOSHELL "dump(\@changes ): " . dump( @changes ); 
+						if ( $tooltype eq "energyplus" ) # RESTORES DEFAULTS FOR ENERGYPLUS
 						{
-							foreach my $change ( @changes )
+							$outputdir = "/Output";
+							$modfiletype = ".idf";
+							unless ( defined( $resfilename ) )
 							{
-								my $source = $change->[0]; say T say TOSHELL "dump(\$source ): " . dump( $source ); 
-								my $target = $change->[1]; say TOSHELL "dump(\$target ): " . dump( $target ); say TOSHELL "dump(\$rowBEFORE ): " . dump( $epnewfile ); 
-								$row =~ s/$source/$target/ ; say TOSHELL "dump(\$rowAFTER ): " . dump( $row ); 
-								print EPNEWFILE $row;
+								$resfilename = ".eso";
 							}
-						}	
-						close EPNEWFILE;
+						}
 						
-						my $file_eplus = "$to/$file_eplus";  say TOSHELL "dump(\$file_eplus ): " . dump( $file_eplus ); 
+						open ( EPOLDPATH, "$epoldpath" ) or die( "$!" );
+						my @sourcecontents = <EPOLDPATH>;
+						close EPOLDPATH; #say TOSHELL "dump(\@sourcecontents ): " . dump( @sourcecontents ); 
 						
-						my $simelt =~ s/$mypath/$mypath\/Output/; say TOSHELL "dump(\$simelt ): " . dump( $simelt ); 
-						my $resfile = "$simelt/$epnewfile.eso"; # ZZZ: CHECK say TOSHELL "dump(\$resfile ): " . dump( $resfile ); 
+						unless ( -e $epnewpath )
+						{
+							open ( EPNEWPATH, ">$epnewpath" ) or die( "$!" );
+							foreach my $row ( @sourcecontents )
+							{
+								foreach my $change ( @changes )
+								{
+									my $source = $change->[0]; #say T say TOSHELL "dump(\$source ): " . dump( $source ); 
+									my $target = $change->[1]; #say TOSHELL "dump(\$target ): " . dump( $target ); say TOSHELL "dump(\$rowBEFORE ): " . dump( $row ); 
+									$row =~ s/$source/$target/ ; #say TOSHELL "dump(\$rowAFTER ): " . dump( $row ); 
+								}
+								print EPNEWPATH $row;
+
+							}	
+							close EPNEWPATH;
+						}
+						
+						my $file_eplus = "$to/$file_eplus";  #say TOSHELL "dump(\$file_eplus ): " . dump( $file_eplus ); 
+						
+						my $simelt = $mypath . "$outputdir"; #say TOSHELL "dump(\$simelt ): " . dump( $simelt ); 
+						my $resfile = "$simelt/$epresroot$resfiletype"; #say TOSHELL "dump(\$resfile ): " . dump( $resfile ); 
 						
 						if (not (-e $simblock ) )
 						{
@@ -413,33 +437,44 @@ $printthis
 							print RETLIST "$resfile\n";
 								
 							unless ( $preventsim eq "y" )
-							{
-										
-								open ( OLDFILEEPLUS, $epoldfile ) or die;
+							{	
+								open ( OLDFILEEPLUS, $epoldpath ) or die( "$!" );
 								my @oldlines = <OLDFILEEPLUS>;
 								close OLDFILEEPLUS;
-								
-								open ( NEWFILEEPLUS, ">$epnewfile" ) or die;
-								
-								foreach my $line ( @oldlines )
+								unless ( -e $epnewpath )
 								{
-									foreach my $elt ( @changes )
+									open ( NEWFILEEPLUS, ">$epnewpath" ) or die( "$!" );
+									foreach my $line ( @oldlines )
 									{
-										my $old = $elt->[0];
-										my $new = $elt->[1];
-										$line =~ s/$old/$new/;
+										foreach my $elt ( @changes )
+										{
+											my $old = $elt->[0];
+											my $new = $elt->[1];
+											$line =~ s/$old/$new/;
+										}
+										print NEWFILEEPLUS $line;
 									}
-									print NEWFILEEPLUS $line;
+									close NEWFILEEPLUS;
 								}
-								close NEWFILEEPLUS;
 								
-								unless ( $exeonfiles eq "n" )
+								my $templaunch;
+								unless ( -e $resfile )
 								{
-									`runenergyplus $epnewfile $epwfile`;
+									unless ( $exeonfiles eq "n" )
+										{
+										my $tempf = $epnewpath; #say TOSHELL "dump(\$tempf1 ): " . dump( $tempf ); 
+										$tempf =~ s/$to$epdir\///;  #say TOSHELL "dump(\$tempf2 ): " . dump( $tempf ); 
+										$templaunch = $mypath . "/" . $tempf; #say TOSHELL "dump(\$templaunch ): " . dump( $templaunch ); 
+										`cp -f $epnewpath $templaunch`;
+										`runenergyplus $templaunch $epwfile`;
+										#`rm -f $templaunch`;
+									}
+									print TOSHELL "cp -f $epnewpath $templaunch\n";
+									print TOSHELL "runenergyplus $templaunch $epwfile\n";
+									#print TOSHELL "rm -f $templaunch\n";
+									say "#Simulating case " . ($countcase + 1) . ", block " . ($countblock + 1) . ", parameter $countvar at iteration $countstep for tool $tooltype. Instance $countinstance: using $epnewpath (actually $templaunch) to obtain $resfile. " ;										
 								}
-								print TOSHELL "runenergyplus $epnewfile $epwfile\n";
-								say "#Simulating case " . ($countcase + 1) . ", block " . ($countblock + 1) . ", parameter $countvar at iteration $countstep for tool $tooltype. Instance $countinstance: writing $resfile. " ;										
-							}		
+							}	
 						}
 						$countsim++;
 					}
@@ -451,6 +486,8 @@ $printthis
 	}
 	close SIMLIST;
 	close SIMBLOCK;
+	say TOSHELL "dump(\@simcases ): " . dump( @simcases ); 
+	say TOSHELL "dump(\@simstruct ): " . dump( @simstruct ); 
 	return ( \@simcases, \@simstruct );
 	close TOSHELL;
 	close OUTFILE;
